@@ -122,12 +122,17 @@ def ke2v(ke, mass=None, mass_units='amu',
     36.6 angstrom/picosecond
 
     """
-    #m = Qty(atoms.m, 'amu').ito('eV/c**2').ito('eV/({})**2'.format(v_units))
     if isinstance(ke, (int, float)):
         ke = Qty(ke, ke_units)
     else:
         ke = Qty(ke.magnitude, ke.units)
-    m = Qty(mass, mass_units).ito('eV/({})**2'.format(v_units))
+
+    if mass is None:
+        raise TypeError('mass must be an int or float')
+
+    m = Qty(mass, mass_units)
+    m.ito('eV/({})**2'.format(v_units))
+
     v = np.sqrt(2 * ke / m)
     if magnitude_only:
         return v.magnitude
@@ -172,7 +177,12 @@ def v2ke(v, mass=None, mass_units='amu',
     else:
         v = Qty(v.magnitude, v.units)
 
-    m = Qty(mass, mass_units).ito('eV/({})**2'.format(v_units))
+    if mass is None:
+        raise TypeError('mass must be an int or float')
+
+    m = Qty(mass, mass_units)
+    m.ito('eV/({})**2'.format(v_units))
+
     ke = 0.5 * m * v**2
     if magnitude_only:
         return ke.magnitude
