@@ -48,52 +48,24 @@ class LogGenerator(object):
 
     def __init__(self, logname=None, logfile=None, logtype=None):
 
-        self._fields = OrderedDict()
-        self._field_names = []
-        self._field_widths = []
-        self._logname = None
-        self._logfile = None
+        self.fields = OrderedDict()
+        self.field_names = []
+        self.field_widths = []
+
+        if logname is None or not isinstance(logname, str):
+            logname = 'log'
 
         logext = logtype_fext[logtype]
-
-        if isinstance(logname, str):
-            self._logname = logname
-        else:
-            self._logname = 'log'
-
         if logfile is None:
             dt = datetime.now()
             timestr = dt.strftime("%Y-%m-%d_%H%M%S")
-            self._logfile = self._logname + '-' + timestr + logext
+            logfile = logname + '-' + timestr + logext
         else:
-            self._logfile = logfile
             if not logfile.endswith(logext):
-                self._logfile += logext
+                logfile += logext
 
-    @property
-    def fields(self):
-        """:py:class:`~python:collections.OrderedDict` of `fields`."""
-        return self._fields
-
-    @property
-    def field_names(self):
-        """list of `field_names`."""
-        return self._field_names
-
-    @property
-    def field_widths(self):
-        """list of `field_widths`."""
-        return self._field_widths
-
-    @property
-    def logfile(self):
-        """logfile."""
-        return self._logfile
-
-    @property
-    def logname(self):
-        """logname."""
-        return self._logname
+        self.logname = logname
+        self.logfile = logfile
 
 
 class TextLogGenerator(LogGenerator):
@@ -115,8 +87,8 @@ class TextLogGenerator(LogGenerator):
     def __init__(self, fields=None, logname=None, logfile=None,
                  manual_flush=True, mode='a+', wspad=4):
 
-        self._manual_flush = manual_flush
-        self._mode = mode
+        self.manual_flush = manual_flush
+        self.mode = mode
 
         if sys.version_info[0] < 3:
             # call super class using python 2 approach
@@ -164,11 +136,11 @@ class TextLogGenerator(LogGenerator):
                 field_dict.update(
                     {'delim': delim_width*'-', 'width': field_width})
 
-                self._fields.update({field_key: field_dict})
-                self._field_names.append(field_name)
-                self._field_widths.append(field_width)
+                self.fields.update({field_key: field_dict})
+                self.field_names.append(field_name)
+                self.field_widths.append(field_width)
 
-        with open(self._logfile, 'a+') as textlog:
+        with open(self.logfile, self.mode) as textlog:
             lines = textlog.readlines()
             if len(lines) == 0:
                 self.write_header()
@@ -176,21 +148,30 @@ class TextLogGenerator(LogGenerator):
     def __str__(self):
         """String representation of `TextLogGenerator`."""
         return "<TextLogGenerator instance: {}.{}>".format(
-            self._logname, self._logfile)
+            self.logname, self.logfile)
 
     def dict2logline(self, d):
         logline = ''
-        for key, field in self._fields.iteritems():
-            try:
-                logline += '{0:>{width}}'.format(d[key], **field)
-            except KeyError:
+        for key, field in self.fields.iteritems():
+            if key in d:
+                try:
+                    logline += \
+                        '{0:>{width}.{precision}{ptype}}'.format(d[key],
+                                                                 **field)
+                except (KeyError, ValueError):
+                    try:
+                        logline += \
+                            '{0:>{width}{ptype}}'.format(d[key], **field)
+                    except (KeyError, ValueError):
+                        logline += '{0!s:>{width}}'.format(d[key], **field)
+            else:
                 logline += '{0:>{width}}'.format(None, **field)
         logline += '\n'
         self.write_line(logline)
 
     def seq2logline(self, seq):
         logline = ''
-        for n, field in enumerate(self._fields.itervalues()):
+        for n, field in enumerate(self.fields.itervalues()):
             logline += '{0:>{width}}'.format(seq[n], **field)
         logline += '\n'
         self.write_line(logline)
@@ -204,13 +185,13 @@ class TextLogGenerator(LogGenerator):
 
         """
         logline = ''
-        for field in self._fields.itervalues():
+        for field in self.fields.itervalues():
             logline += '{name!s:>{width}}'.format(**field)
         logline += '\n'
         self.write_line(logline, textlog=textlog)
 
         logline = ''
-        for field in self._fields.itervalues():
+        for field in self.fields.itervalues():
             logline += '{delim!s:>{width}}'.format(**field)
         logline += '\n'
         self.write_line(logline, textlog=textlog)
@@ -240,18 +221,18 @@ class TextLogGenerator(LogGenerator):
 
         """
         if textlog is None:
-            with open(self._logfile, self._mode) as textlog:
+            with open(self.logfile, self.mode) as textlog:
                 textlog.write(logline)
-                if self._manual_flush:
+                if self.manual_flush:
                     textlog.flush()
                     os.fsync(textlog.fileno())
         elif isinstance(textlog, TextLogGenerator):
             textlog.write_line(logline)
         elif isinstance(textlog, file):
             textlog.write(logline)
-            if self._manual_flush:
+            if self.manual_flush:
                 textlog.flush()
                 os.fsync(textlog.fileno())
         else:
-            raise TextLogGeneratorError(
-                "Can't write to textlog: {}".format(textlog))
+            raise TextLogGeneratorError("Can't write to textlog: {}".format(
+                textlog))
