@@ -47,7 +47,7 @@ class PlotGenerator(object):
     """
     def __init__(self, datasets=None, plotconfig=None, argparser=None,
                  plotcfgparser=None, plotcfgfile=None, insetcfgfile=None,
-                 overlaycfgfile=None):
+                 overlaycfgfile=None, verbose=False):
         self._datasets = datasets
         self._plotconfig = plotconfig
         self._argparser = argparser
@@ -55,6 +55,7 @@ class PlotGenerator(object):
         self._plotcfgfile = plotcfgfile
         self._insetcfgfile = insetcfgfile
         self._overlaycfgfile = overlaycfgfile
+        self._verbose = verbose
 
     @property
     def datasets(self):
@@ -104,26 +105,16 @@ class XYPlotGenerator(PlotGenerator):
             self._datasets = self._plotcfgparser.datasets
             self._plotconfig = self._plotcfgparser.config
 
-        #args = self._plotcfgparser.parsed_args
-        #XYPlot.plot(datasets, plotconfig, **vars(args))
-
-        if hasattr(self._plotconfig, 'plotgrid'):
-            self._xyplot = XYGridPlot(self._plotconfig.plotgrid,
-                                      self._datasets,
-                                      self._plotconfig)
-        else:
-            self._xyplot = XYPlot(self._datasets, self._plotconfig)
+        self._xyplot = XYPlot(self._datasets, self._plotconfig)
 
         if autoplot:
             self._xyplot.plot()
 
-            XYPlot.set_axes_labels(plotconfig=self._plotconfig,
-                                   axes=self._xyplot.axes,
-                                   figkwargs=self._xyplot.figkwargs)
-
             XYPlot.savefig(plotconfig=self._plotconfig,
                            figure=self._xyplot.figure,
-                           figkwargs=self._xyplot.figkwargs)
+                           figkwargs=self._xyplot.figkwargs,
+                           nrows=self._xyplot.nrows,
+                           ncols=self._xyplot.ncols)
 
     @property
     def xyplot(self):
@@ -132,8 +123,7 @@ class XYPlotGenerator(PlotGenerator):
 
 class MPLPlot(object):
     """Base class for matplotlib plots."""
-    def __init__(self, datasets=None, plotconfig=None, figure=None, axes=None,
-                 rows=1, cols=1):
+    def __init__(self, datasets=None, plotconfig=None, figure=None, axes=None):
         self._datasets = datasets
         self._plotconfig = plotconfig
         self._figure = figure
@@ -146,14 +136,34 @@ class MPLPlot(object):
                              fontfamily=plotconfig.fontfamily,
                              aspectratio=plotconfig.aspectratio)
 
+        self._multicol = self._multirow = False
+        self._nrows = self._ncols = 1
+
         if axes is None:
-            if rows == 1 and cols == 1:
-                squeeze = True
-            else:
-                squeeze = False
+            self._nrows = self._plotconfig.nrows
+            self._ncols = self._plotconfig.ncols
+            self._sharex = self._plotconfig.sharex
+            self._sharey = self._plotconfig.sharey
+
+            for axis in ('x', 'y'):
+                share_option = getattr(self, '_share' + axis)
+                if isinstance(share_option, str):
+                    if share_option == 'none':
+                        setattr(self, '_share' + axis, False)
+                    elif share_option == 'all':
+                        setattr(self, '_share' + axis, True)
+
+            if self._nrows == 1 and self._ncols > 1:
+                self._multicol = True
+            elif self._nrows > 1 and self._ncols == 1:
+                self._multirow = True
+            elif self._nrows > 1 and self._ncols > 1:
+                self._multicol = self._multirow = True
 
             self._figure, self._axes = \
-                plt.subplots(rows, cols, squeeze=squeeze)
+                plt.subplots(nrows=self._nrows, ncols=self._ncols,
+                             sharex=self._sharex, sharey=self._sharey,
+                             squeeze=self._plotconfig.squeeze)
 
     @property
     def figure(self):
@@ -187,6 +197,14 @@ class MPLPlot(object):
     def mpl(self):
         del self._mpl
 
+    @property
+    def nrows(self):
+        return self._nrows
+
+    @property
+    def ncols(self):
+        return self._ncols
+
 
 class XYPlot(MPLPlot):
     """Make 2D XY plot.
@@ -217,93 +235,138 @@ class XYPlot(MPLPlot):
 
         for dataset_id, dataset in self._datasets.iteritems():
 
-            if dataset.ydata is None:
-                dataset.ydata = \
-                    dataset.data[:, dataset.headers.index(
-                        self._plotconfig.input.headers.yvar)] * \
-                    self._plotconfig.datasets.yscale_factor
-
-            if dataset.xdata is None:
-                if self._plotconfig.input.headers.xvar is not None:
-                    dataset.xdata = \
-                        dataset.data[:, dataset.headers.index(
-                            self._plotconfig.input.headers.xvar)] * \
-                        self._plotconfig.datasets.yscale_factor
-                else:
-                    dataset.xdata = \
-                        np.arange(0, len(dataset.ydata)) * \
-                        self._plotconfig.datasets.xscale_factor
-
-            xdata = dataset.xdata
-            ydata = dataset.ydata
-
-            xmax = max(xmax, xdata.max())
-            xmin = min(xmin, xdata.min())
-            ymax = max(ymax, ydata.max())
-            ymin = min(ymin, ydata.min())
-
             dataset_plotconfig = \
                 getattr(self._plotconfig.datasets, dataset_id)
-            p1, = self._axes.plot(xdata, ydata,
-                                  marker=dataset_plotconfig.marker,
-                                  color=dataset_plotconfig.markercolor,
-                                  markersize=dataset_plotconfig.markersize,
-                                  mec=dataset_plotconfig.markeredgecolor,
-                                  mew=dataset_plotconfig.markeredgewidth,
-                                  mfc=dataset_plotconfig.markerfacecolor,
-                                  fillstyle=dataset_plotconfig.fillstyle,
-                                  ls=dataset_plotconfig.linestyle,
-                                  lw=dataset_plotconfig.linewidth,
-                                  label=dataset_plotconfig.label,
-                                  alpha=dataset_plotconfig.alpha,
-                                  markevery=dataset_plotconfig.markevery,
-                                  solid_joinstyle='round')
 
-            plots.append(p1)
-            offsets.append((self._figkwargs['dropshadow.offset.x'] / 2,
-                            self._figkwargs['dropshadow.offset.y'] / 2))
+            axes = self._axes
+            if not self._multirow and self._multicol:
+                axes = self._axes[dataset_plotconfig.col - 1]
+            elif self._multirow and not self._multicol:
+                axes = self._axes[dataset_plotconfig.row - 1]
+            elif self._multirow and self._multicol:
+                axes = self._axes[dataset_plotconfig.row - 1,
+                                  dataset_plotconfig.col - 1]
 
-        if self._plotconfig.add_image_overlay:
-            XYPlot.add_overlay(figure=self._figure,
-                               overlay_config=self._plotconfig.overlay_config)
+            if dataset.axhline:
+                ymax = max(ymax, dataset.ydata)
+                ymin = min(ymin, dataset.ydata)
+                axes.axhline(y=dataset.ydata,
+                             lw=dataset_plotconfig.linewidth,
+                             c=dataset_plotconfig.markercolor,
+                             label=dataset_plotconfig.label,
+                             alpha=dataset_plotconfig.alpha)
+            elif dataset.axvline:
+                xmax = max(xmax, dataset.xdata)
+                xmin = min(xmin, dataset.xdata)
+                axes.axvline(x=dataset.xdata,
+                             lw=dataset_plotconfig.linewidth,
+                             c=dataset_plotconfig.markercolor,
+                             label=dataset_plotconfig.label,
+                             alpha=dataset_plotconfig.alpha)
+            else:
 
-        if self._plotconfig.xaxis.log_scale:
-            self._axes.set_xscale('log')
-        if self._plotconfig.yaxis.log_scale:
-            self._axes.set_yscale('log')
+                xdata = dataset.xdata * self._plotconfig.datasets.xscale_factor
+                ydata = dataset.ydata * self._plotconfig.datasets.yscale_factor
+
+                xmax = max(xmax, xdata.max())
+                xmin = min(xmin, xdata.min())
+                ymax = max(ymax, ydata.max())
+                ymin = min(ymin, ydata.min())
+
+                p1, = axes.plot(xdata, ydata,
+                                marker=dataset_plotconfig.marker,
+                                color=dataset_plotconfig.markercolor,
+                                markersize=dataset_plotconfig.markersize,
+                                mec=dataset_plotconfig.markeredgecolor,
+                                mew=dataset_plotconfig.markeredgewidth,
+                                mfc=dataset_plotconfig.markerfacecolor,
+                                fillstyle=dataset_plotconfig.fillstyle,
+                                ls=dataset_plotconfig.linestyle,
+                                lw=dataset_plotconfig.linewidth,
+                                label=dataset_plotconfig.label,
+                                alpha=dataset_plotconfig.alpha,
+                                markevery=dataset_plotconfig.markevery,
+                                solid_joinstyle='round')
+
+                plots.append(p1)
+                offsets.append((self._figkwargs['dropshadow.offset.x'] / 2,
+                                self._figkwargs['dropshadow.offset.y'] / 2))
+
+        #if self._plotconfig.add_image_overlay:
+        #    XYPlot.add_overlay(figure=self._figure,
+        #                       overlay_config=self._plotconfig.overlay_config)
+
+        #if self._plotconfig.xaxis.log_scale:
+        #    axes.set_xscale('log')
+        #if self._plotconfig.yaxis.log_scale:
+        #    axes.set_yscale('log')
 
         datalimits = {'xaxis': {'left': xmin, 'right': xmax},
                       'yaxis': {'bottom': ymin, 'top': ymax}}
-        XYPlot.set_axes_limits(axes=self._axes,
-                               plotconfig=self._plotconfig,
-                               datalimits=datalimits)
 
-        if not self._plotconfig.disable_legend:
-            XYPlot.add_legend(axes=self._axes, plotconfig=self._plotconfig)
+        if not self._multirow and self._multicol:
+            self._figure.subplots_adjust(wspace=0)
+            plt.setp([ax.get_yticklabels() for ax in self._figure.axes[1:]],
+                     visible=False)
+            [ax.tick_params(axis='both', which='both', direction='inout')
+             for ax in self._axes]
+        elif self._multirow and not self._multicol:
+            pass
+        elif self._multirow and self._multicol:
+            pass
+        else:
+            XYPlot.set_axes_limits(axes=self._axes,
+                                   plotconfig=self._plotconfig,
+                                   datalimits=datalimits)
+            self._axes.tick_params(
+                axis='both', which='both', direction='inout')
 
-        if self._plotconfig.add_zoomed_inset:
-            XYPlot.add_zoomed_inset(axes=self._axes,
-                                    datasets=self._datasets,
-                                    figkwargs=self._figkwargs,
-                                    plotconfig=self._plotconfig,
-                                    inset_config=self._plotconfig.inset_config)
+            XYPlot.set_axes_labels(axes=self._axes,
+                                   plotconfig=self._plotconfig,
+                                   figkwargs=self._figkwargs)
+
+            if not self._plotconfig.disable_legend:
+                if self._plotconfig.multi_legend:
+                    pass
+                else:
+                    XYPlot.add_legend(axes=self._axes,
+                                      plotconfig=self._plotconfig)
+
+        #if self._plotconfig.add_zoomed_inset:
+        #    XYPlot.add_zoomed_inset(
+        #        axes=self._axes, datasets=self._datasets,
+        #        figkwargs=self._figkwargs, plotconfig=self._plotconfig,
+        #        inset_config=self._plotconfig.inset_config)
 
         if self._plotconfig.enable_dropshadow:
-            plot_dropshadow(self._axes, plots, offsets,
-                            offset_units='points',
+            plot_dropshadow(self._axes, plots, offsets, offset_units='points',
                             gaussian_blur_sigma=4, alpha=0.5)
 
     @classmethod
     def add_legend(cls, axes=None, plotconfig=None):
-        axes.legend(bbox_to_anchor=plotconfig.legend_bbox_to_anchor,
-                    loc=plotconfig.legend_loc,
+        axes.legend(loc=plotconfig.legend_loc,
                     fontsize=plotconfig.legend_fontsize,
-                    frameon=plotconfig.frame_legend,
-                    fancybox=plotconfig.fancy_legend_frame,
-                    framealpha=plotconfig.legend_frame_alpha,
-                    handlelength=plotconfig.legend_handlelength,
+                    numpoints=plotconfig.legend_numpoints,
+                    scatterpoints=plotconfig.legend_scatterpoints,
+                    markerscale=plotconfig.legend_markerscale,
+                    frameon=plotconfig.legend_frameon,
+                    fancybox=plotconfig.legend_fancybox,
+                    shadow=plotconfig.legend_shadow,
+                    framealpha=plotconfig.legend_framealpha,
                     ncol=plotconfig.legend_ncol,
-                    mode=plotconfig.legend_mode)
+                    mode=plotconfig.legend_mode,
+                    bbox_to_anchor=plotconfig.legend_bbox_to_anchor,
+                    title=plotconfig.legend_title,
+                    borderpad=plotconfig.legend_borderpad,
+                    labelspacing=plotconfig.legend_labelspacing,
+                    handlelength=plotconfig.legend_handlelength,
+                    handletextpad=plotconfig.legend_handletextpad,
+                    borderaxespad=plotconfig.legend_borderaxespad,
+                    columnspacing=plotconfig.legend_columnspacing)
+
+    @classmethod
+    def make_legend(cls, plotlist=[], labellist=[], loc='best', title=None):
+        return plt.legend(plotlist, labellist, loc=loc, title=title)
 
     @classmethod
     def add_overlay(cls, figure=None, overlay_config=None):
@@ -356,8 +419,8 @@ class XYPlot(MPLPlot):
                    loc1=2, loc2=4, fc="none", ec="0.5")
 
     @classmethod
-    def savefig(cls, fname=None, plotconfig=None, figure=None,
-                figkwargs=None, verbose=True):
+    def savefig(cls, fname=None, plotconfig=None, figure=None, figkwargs=None,
+                nrows=1, ncols=1, verbose=True):
         """Save figure.
 
         Parameters
@@ -373,16 +436,20 @@ class XYPlot(MPLPlot):
         if fname is None:
             fname = plotconfig.output.fname
 
-        fout = get_fpath(fname, ext=plotconfig.plotformat,
+        fout = get_fpath(fname, ext=plotconfig.output.plotformat,
                          outpath=plotconfig.output.outpath,
                          overwrite=plotconfig.output.overwrite,
                          add_fnum=True, fnum=plotconfig.output.file_num,
                          verbose=verbose)
-        figure.set_size_inches(figkwargs['xdim'], figkwargs['ydim'])
-        figure.savefig(fout, transparent=plotconfig.save_transparent,
-                       format=plotconfig.plotformat,
-                       bbox_inches=plotconfig.bbox_inches,
-                       pad_inches=plotconfig.pad_inches,
+
+        xdim = ncols * figkwargs['xdim']
+        ydim = nrows * figkwargs['ydim']
+
+        figure.set_size_inches(xdim, ydim)
+        figure.savefig(fout, transparent=plotconfig.output.save_transparent,
+                       format=plotconfig.output.plotformat,
+                       bbox_inches=plotconfig.output.bbox_inches,
+                       pad_inches=plotconfig.output.pad_inches,
                        frameon=False)
 
     @classmethod
